/*
 * thermald - pid.h
 *
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <pid.h>
#include <stdio.h>

int pid_init(struct pid *pid, double k, double Ti, double Td, double N)
{
	pid->k = k;
	pid->Ti = Ti;
	pid->Td = Td;
	pid->N = N;
	pid->Tp = 1;

	pid->e1 = 0;
	pid->e2 = 0;
	pid->u = 0;
	return 0;
}

int pid_set_interval(struct pid *pid, double Tp)
{
	pid->Tp = Tp;
	return 0;	
}

int pid_step(struct pid *pid, double e)
{
	double dp = pid->k * (e - pid->e1);
	double di = pid->Tp * pid->k * pid->e1 / pid->Ti;
	double a = pid->Td / (pid->Td + pid->Tp * pid->N);
	double b = pid->k * pid->N * a;
	double dd = b / (1 - a) * (e - 2 * pid->e1 + pid->e2);

	pid->u += dp + di + dd;
	
	/* saturate to [-1; 1] */
	if (pid->u > 1) {
		pid->u = 1;
	} else if (pid->u < -1) {
		pid->u = -1;
	}

	pid->e2 = pid->e1;
	pid->e1 = e;
	return 0;
}

double pid_output(struct pid *pid)
{
	return pid->u;
}
